#!/usr/bin/env python3

config_raw = r'''# NOTE: on windows backslash needs to be escaped like so:
# C:\\my path\\to\\input

# Place your scanned images here
input: "/path/to/input/"

# Your sliced images will be created here
output: "/path/to/output/"

# Skip the confirmation
skip-confirm: False

# Match with your scanners natural white color (0-255)
white-threshold: 230

# Min size of the sliced image in % compared to scanned image area (0-100)
minimum-size: 3

# Use test mode (True/False)
test-mode: False

# Use preview mode (True/False)
preview-mode: False

# Use slice mode (True/False)
slice-mode: False

# Height of test/preview mode image
view-height: 800

# Scale using factor value (0 = disabled)
scale-factor: 0

# Scale using width value in pixels
scale-width: 0

# Scale using height value in pixels
scale-height: 0

# Filter out scanner noise (0-5)
filter-denoise: 1

# Add brightness (values over 1.0)
filter-brightness: 1.0

# Add contrast (values over 1.0)
filter-contrast: 1.0

# Add gamma correction (values over 1.0)
filter-gamma: 1.0

# Save format (PNG|JPEG|WEBP)
save-format: "PNG"

# Level of PNG compression (0-9)
png-compression: 3

# Level of JPEG quality (0-100)
jpeg-quality: 95

# Level of WEBP quality (1-101)
# 101 = lossless
webp-quality: 90
'''



#!/usr/bin/env python3

import sys
import logging as log

from .scis import ScanImageSlicer
from .confparser import conf_parser

LOG_LEVEL = log.INFO
PROGRAM_NAME = "Scan Image Slicer"

def main():
    if LOG_LEVEL != log.INFO:
        lname = '[%(levelname)s] '
    else:
        lname = ":: "

    log.basicConfig(format=lname + '%(message)s', level=LOG_LEVEL, handlers=[log.StreamHandler(sys.stdout)])
    log.info("%s", PROGRAM_NAME)

    parsed_dicts = conf_parser()
    settings = parsed_dicts[0]
    commands = parsed_dicts[1]

    scis = ScanImageSlicer(settings)

    if commands["list_scans"]:
        scis.list_scanned_images()

    if commands["list_file"]:
        scis.list_scanned_images_to_file()

    if commands["add_all"]:
        scis.add_task_all_images()

    if commands["add_id"]:
        scis.add_task_with_image_id(commands["add_id"])

    if commands["add_new"]:
        scis.add_task_newest_n_images(commands["add_new"])

    if commands["add_old"]:
        scis.add_task_oldest_n_images(commands["add_old"])

    if commands["add_random"]:
        scis.add_task_random_n_images(commands["add_random"])

    if commands["remove_id"]:
        scis.remove_task_with_image_id(commands["remove_id"])

    if commands["list_cmds"]:
        scis.list_commands()

    if commands["list_tasks"]:
        scis.list_tasks()

    if settings["slice_mode"] or settings["test_mode"] or settings["preview_mode"]:
        scis.start(settings["skip_confirm"])

if __name__ == "__main__":
    sys.exit(main())


#!/usr/bin/env python3

import os
import sys
import logging as log
import configargparse
from .config import config_raw

def conf_parser():

    if sys.platform in ["win32", "cygwin"]:
        path_config = os.path.join(os.path.expanduser("~"), "Documents", "ScanImageSlicer")
    else:
        path_config = os.path.join(os.path.expanduser("~"), ".config", "ScanImageSlicer")
        
    if not os.path.exists(path_config):
        os.mkdir(path_config)

    path_config_file = os.path.join(path_config, "config.yaml")
    if not os.path.isfile(path_config_file):
        with open(path_config_file, 'w') as outfile:
            try:
                outfile.writelines(config_raw)
                outfile.close()
            except OSError as e:
                sys.exit(e)

        if os.path.isfile(path_config_file):
            log.info("Created default config file: %s", path_config_file)

    line = ""
    n = "\n"
    title = "  Scan Image Slicer: Detect and slice images from a scanned image (or any image)  "
    for c in title:
        line += "-"
    info = f"  Config file at: {path_config_file}"

    description = line + n + n + title + n + info + n + n + line

    parser = configargparse.ArgumentParser(
        default_config_files=[path_config_file],
        description=description,
        config_file_parser_class=configargparse.YAMLConfigFileParser,
        formatter_class=configargparse.RawTextHelpFormatter,
        allow_abbrev=False)

    parser.add_argument("-c", "--config_file", metavar="FILE",
                        is_config_file=True, help="path to custom config file")

    parser.add_argument("-skip", "--skip-confirm", action="store_true",
                        help="skip the confirm start question")

    mode_group = parser.add_argument_group("settings|modes")
    mode_select_group = mode_group.add_mutually_exclusive_group()

    mode_select_group.add_argument("-test", "--test-mode", action="store_true",
                        help="enable test mode")

    mode_select_group.add_argument("-pre", "--preview-mode", action="store_true",
                        help="enable preview mode")

    mode_select_group.add_argument("-slice", "--slice-mode", action="store_true",
                        help="enable slice mode")

    path_group = parser.add_argument_group("settings|paths")

    path_group.add_argument("-i", "--input", metavar="PATH",
                        type=str, help="input folder for the scanned images")

    path_group.add_argument("-o", "--output", metavar="PATH",
                        type=str, help="output folder for the sliced images")

    detect_group = parser.add_argument_group("settings|image detection")

    detect_group.add_argument("-white", "--white-threshold", metavar="0..255",
                        type=int, help="a value used for image detection")
                        
    detect_group.add_argument("-min", "--minimum-size", metavar="0..100",
                        type=int, help="a value used for discarding too small (false) images")

    detect_group.add_argument("-view", "--view-height", metavar="N",
                        type=int, help="the height of the image inside the image viewer")

    scale_group = parser.add_argument_group("settings|scaling")

    scale_group.add_argument("-scaleF", "--scale-factor", metavar="F",
                        type=float, help="scale sliced image using a scale factor value")
                        
    scale_group.add_argument("-scaleW", "--scale-width", metavar="N",
                        type=int, help="scale sliced image using a new width value")
                        
    scale_group.add_argument("-scaleH", "--scale-height", metavar="N",
                        type=int, help="scale sliced image using a new height value")

    filter_group = parser.add_argument_group("settings|filters")

    filter_group.add_argument("-filtD", "--filter-denoise", metavar="0..5",
                        type=int, help="remove scanner noise from the sliced image")

    filter_group.add_argument("-filtB", "--filter-brightness", metavar="N > 1.0",
                        type=float, help="add brightness to the sliced image")

    filter_group.add_argument("-filtC", "--filter-contrast", metavar="N > 1.0",
                        type=float, help="add contrast to the sliced image")

    filter_group.add_argument("-filtG", "--filter-gamma", metavar="N > 1.0",
                        type=float, help="add gamma correction to the sliced image")

    io_group = parser.add_argument_group("settings|file format")

    io_group.add_argument("-save", "--save-format", metavar="JPEG|PNG|WEBP",
                        type=str, help="the file format of the sliced image")
                        
    io_group.add_argument("-png", "--png-compression", metavar="0..9",
                        type=int, help="PNG compression level")

    io_group.add_argument("-jpeg", "--jpeg-quality", metavar="0..100",
                        type=int, help="JPEG quality level")

    io_group.add_argument("-webp", "--webp-quality", metavar="1..101",
                        type=int, help="WEBP quality level")
 
    list_group = parser.add_argument_group("commands|lists")

    list_group.add_argument("-listS", "--list-scans", action="store_true",
                        default=False, help="list scanned images name and ID")

    list_group.add_argument("-listF", "--list-file", action="store_true",
                        default=False, help="save scanned images name and ID as a text file")

    list_group.add_argument("-listT", "--list-tasks", action="store_true",
                        default=False, help="list added tasks")

    list_group.add_argument("-listC", "--list-cmds", action="store_true",
                        default=False, help="list given commands")

    task_group = parser.add_argument_group("commands|tasks")

    task_group.add_argument("-addA", "--add-all", action="store_true",
                        default=False, help="add all scanned images")

    task_group.add_argument("-addID", "--add-id", nargs="+",
                        metavar="ID", type=int, help="add scanned images using IDs eg. 1 2 3")

    task_group.add_argument("-addN", "--add-new", metavar="N", 
                        type=int, help="add N amount of newest scanned images by creation time (ctime)")

    task_group.add_argument("-addO", "--add-old", metavar="N", 
                        type=int, help="add N amount of oldest scanned images by creation time (ctime)")

    task_group.add_argument("-addR", "--add-random", metavar="N", 
                        type=int, help="add N amount of random scanned images")

    task_group.add_argument("-remID", "--remove-id", nargs="+",
                        metavar="ID", type=int, help="remove scanned images using IDs eg. 1 2 3")

    args = parser.parse_args()

    settings = {}
    errors = []
    
    if args.skip_confirm:
        settings["skip_confirm"] = True
    else:
        settings["skip_confirm"] = False

    if os.path.exists(args.input):
        if args.input == "":
            errors.append("Use a valid input path")
        else:
            settings["input"] = os.path.normpath(args.input)
    else:
        errors.append(f"Input path does not exist: {args.input}")

    if os.path.exists(args.output):
        if args.output == "":
            errors.append("Use a valid output path")
        else:
            settings["output"] = os.path.normpath(args.output)
    else:
        errors.append(f"Output path does not exist: {args.output}")

    if args.white_threshold in range(0, 256):
        settings["white_threshold"] = args.white_threshold
    else:
        errors.append("Value of white_threshold should be between 0 and 255")

    if args.minimum_size in range(0, 101):
        settings["minimum_size"] = args.minimum_size
    else:
        errors.append("Value of minimum_size should be between 0 and 100")

    if args.test_mode:
        settings["test_mode"] = args.test_mode
    else:
        settings["test_mode"] = False

    if args.preview_mode:
        settings["preview_mode"] = args.preview_mode
    else:
        settings["preview_mode"] = False

    if args.slice_mode:
        settings["slice_mode"] = args.slice_mode
    else:
        settings["slice_mode"] = False

    if args.view_height > 0:
        settings["view_height"] = args.view_height
    else:
        errors.append("Value of view_height must be positive int value")

    if args.scale_factor != 0:
        settings["scale_factor"] = args.scale_factor
    else:
        settings["scale_factor"] = 0

    if args.scale_width != 0:
        settings["scale_width"] = args.scale_width
    else:
        settings["scale_width"] = 0

    if args.scale_height != 0:
        settings["scale_height"] = args.scale_height
    else:
        settings["scale_height"] = 0

    if args.filter_denoise in range(0, 6):
        settings["filter_denoise"] = args.filter_denoise
    else:
        errors.append("Value of filter_denoise should be between 0 and 5")

    if args.filter_brightness != 1.0:
        settings["filter_brightness"] = args.filter_brightness
    else:
        settings["filter_brightness"] = 1.0

    if args.filter_contrast != 1.0:
        settings["filter_contrast"] = args.filter_contrast
    else:
        settings["filter_contrast"] = 1.0

    if args.filter_gamma != 1.0:
        settings["filter_gamma"] = args.filter_gamma
    else:
        settings["filter_gamma"] = 1.0

    if args.save_format in ["JPEG", "PNG", "WEBP"]:
        settings["save_format"] = args.save_format
    else:
        errors.append("Save format should be one of JPEG, PNG or WEBP")

    if args.png_compression in range(0, 10):
        settings["png_compression"] = args.png_compression
    else:
        errors.append("Value of png_compression should be between 0 and 9")

    if args.jpeg_quality in range(0, 101):
        settings["jpeg_quality"] = args.jpeg_quality
    else:
        errors.append("Value of jpeg_quality should be between 0 and 100")

    if args.webp_quality in range(1, 102):
        settings["webp_quality"] = args.webp_quality
    else:
        errors.append("Value of webp_quality should be between 1 and 101")

    commands = {}

    if args.list_scans:
        commands["list_scans"] = True
    else:
        commands["list_scans"] = False

    if args.list_file:
        commands["list_file"] = True
    else:
        commands["list_file"] = False

    if args.list_tasks:
        commands["list_tasks"] = True
    else:
        commands["list_tasks"] = False

    if args.list_cmds:
        commands["list_cmds"] = True
    else:
        commands["list_cmds"] = False

    if args.add_all:
        commands["add_all"] = True
    else:
        commands["add_all"] = False

    if type(args.add_id) is list:
        commands["add_id"] = args.add_id
    else:
        commands["add_id"] = False

    if type(args.add_new) is int:
        commands["add_new"] = args.add_new
    else:
        commands["add_new"] = False

    if type(args.add_old) is int:
        commands["add_old"] = args.add_old
    else:
        commands["add_old"] = False

    if type(args.add_random) is int:
        commands["add_random"] = args.add_random
    else:
        commands["add_random"] = False

    if type(args.remove_id) is list:
        commands["remove_id"] = args.remove_id
    else:
        commands["remove_id"] = False

    if errors:
        print()
        log.info("Please fix the following errors:")
        for error in errors:
            log.error(error)
        sys.exit()

    return [settings, commands]


#!/usr/bin/env python3

import os
import sys
import logging as log
import cv2 as cv
import imutils as im
import random

from operator import itemgetter
# from imutils import perspective
from tqdm.auto import tqdm
from time import strftime, localtime, time_ns, gmtime
from .filters import denoise, bcg, show_images

class ScanImageSlicer:
    def __init__(self, settings):
        self.settings = settings
        self.scanned_images = self.collect_scanned_images()
        self.tasks = []
        self.cmds = []
        self.dots = ".........."
        self.output_counter = {}

        inpath = os.path.split(self.settings["input"])
        outpath = os.path.split(self.settings["output"])

        if inpath[1] == "":
            self.settings["input"] = inpath[0]
        else:
            self.settings["input"] = os.path.join(inpath[0], inpath[1])

        if outpath[1] == "":
            self.settings["output"] = outpath[0]
        else:
            self.settings["output"] = os.path.join(outpath[0], outpath[1])

    def slice(self, scanned_image):
        image_count = 0
        false_image_count = 0
        log.debug("Processing scanned image: %s", scanned_image[0])

        try:
            mat_source = cv.imread(scanned_image[1])

            if mat_source.shape[0] > 0:
                pass

        except AttributeError:
            log.error("Could not open file at: %s", scanned_image[1])
            sys.exit()

        image_area_px = mat_source.data.shape[0] * mat_source.shape[1]

        res_width = min(900, mat_source.shape[1])

        mat_res = im.resize(mat_source, width=res_width)
        mat_thresh = cv.cvtColor(mat_res, cv.COLOR_BGR2GRAY)
        mat_thresh = cv.GaussianBlur(mat_thresh, (5, 5), 0)
        mat_thresh = cv.threshold(mat_thresh,
                                  self.settings["white_threshold"],
                                  255,
                                  cv.THRESH_BINARY_INV)[1]

        contours = cv.findContours(mat_thresh, cv.RETR_EXTERNAL, cv.CHAIN_APPROX_SIMPLE)
        contours = im.grab_contours(contours)

        if self.settings["test_mode"]:
            for contour in contours:
                contour_area_pct = (100 / image_area_px * cv.contourArea(contour)) * 100
                x, y, w, h = cv.boundingRect(contour)
                blue = (255, 33, 33)
                purple = (255, 33, 255)
                border = 2

                if contour_area_pct > self.settings["minimum_size"]:
                    cv.rectangle(mat_res, (x, y), (x + w, y + h), blue, border)
                    image_count += 1
                else:
                    cv.rectangle(mat_res, (x, y), (x + w, y + h), purple, border)
                    false_image_count += 1

            log.debug("%s: %s images (%s real, %s false)",
                    "|" + scanned_image[0],
                    image_count + false_image_count,
                    image_count,
                    false_image_count)

            if show_images(scanned_image[0],
                                list([mat_res]),
                                self.settings["view_height"],
                                False) == "exit":
                sys.exit()

        elif self.settings["slice_mode"] or self.settings["preview_mode"]:
            for contour in contours:
                contour_area_pct = (100 / image_area_px * cv.contourArea(contour)) * 100

                if contour_area_pct > self.settings["minimum_size"]:
                    x_fac = mat_source.shape[1] / mat_res.shape[1]
                    y_fac = mat_source.shape[0] / mat_res.shape[0]
                    contour[:,:,0] = contour[:,:,0] * x_fac
                    contour[:,:,1] = contour[:,:,1] * y_fac

                    x, y, w, h = cv.boundingRect(contour)

                    mat_save = mat_source[y:min(y + h, mat_source.shape[0]),
                                          x:min(x + w, mat_source.shape[1])]

                    scaled = False

                    if self.settings["scale_factor"] != 0:
                        scale_factor = self.settings["scale_factor"]
                        mat_save = cv.resize(mat_save, None, fx=scale_factor, fy=scale_factor, interpolation=cv.INTER_AREA)
                        scaled = True

                    if not scaled and self.settings["scale_width"] != 0:
                        scale_width = self.settings["scale_width"]
                        mat_save = im.resize(mat_save, width=scale_width)
                        scaled = True

                    if not scaled and self.settings["scale_height"] != 0:
                        scale_height = self.settings["scale_height"]
                        mat_save = im.resize(mat_save, height=scale_height)

                    if self.settings["filter_denoise"] != 0:
                        mat_filt = denoise(mat_save, self.settings["filter_denoise"])

                    if self.settings["filter_brightness"] != 1.0 or \
                       self.settings["filter_contrast"] != 1.0 or \
                       self.settings["filter_gamma"] != 1.0:
                        mat_filt = bcg(mat_filt, self.settings["filter_brightness"],
                                                 self.settings["filter_contrast"],
                                                 self.settings["filter_gamma"])

                    action = ""
                    if self.settings["preview_mode"]:
                            action = show_images("|S to save|F to toggle filters",
                                                  list([mat_save, mat_filt]),
                                                  self.settings["view_height"],
                                                  True)

                    if action == "exit":
                        sys.exit()

                    if action == "save" or self.settings["slice_mode"]:

                        if not os.path.exists(scanned_image[2]):
                            os.makedirs(scanned_image[2])

                        file_name = ""
                        file_name_divider = "-"
                        inpath = self.settings["input"]
                        outpath = self.settings["output"]
                        abs_path = scanned_image[1]
                        dest_path = scanned_image[2]
                        rel_path = scanned_image[3]

                        if rel_path == ".":
                            file_name += os.path.split(inpath)[1] + file_name_divider
                        else:
                            parts = rel_path.split(os.sep)
                            
                            for part in parts:
                                file_name += part + file_name_divider

                        if file_name in self.output_counter.keys():
                            self.output_counter[file_name] += 1
                        else:
                            self.output_counter[file_name] = 1

                        file_name += str(self.output_counter[file_name])
    
                        if self.settings["save_format"] == "PNG":
                            file_name += ".png"
                            params = [int(cv.IMWRITE_PNG_COMPRESSION), self.settings["png_compression"]]
                        elif self.settings["save_format"] == "JPEG":
                            file_name += ".jpg"
                            params = [int(cv.IMWRITE_JPEG_QUALITY), self.settings["jpeg_quality"]]
                        elif self.settings["save_format"] == "WEBP":
                            file_name += ".webp"
                            params = [int(cv.IMWRITE_WEBP_QUALITY), self.settings["webp_quality"]]

                        outfile = os.path.normpath(os.path.join(dest_path, file_name))
                        
                        cv.imwrite(outfile, mat_filt, params)
                        image_count += 1

                        log.debug("Saved file: %s", outfile)
        else:
            log.error("Unknown mode")
            sys.exit()

        return image_count

    def start(self, skip_confirm):
        print()
        start = False
        sliced_images_count = 0
        log.info("Starting slicer with these settings:")
        for option in self.settings:
            log.info("%s: %s", option, self.settings[option])
        print()
        log.info("Tasks to complete: %s", len(self.tasks))
        if not self.tasks:
            log.info("No tasks were completed")
            sys.exit()

        if not skip_confirm:
            log.info("Continue? y/n")
            
            while True:
                try:
                    answer = input(":: Answer: ").lower()

                    if answer == 'y':
                        start = True
                        break
                    elif answer == 'n':
                        print()
                        log.info("Aborting")
                        sys.exit()
                    else:
                        continue
                except OSError as e:
                    sys.exit(e)
        else:
            start = True

        if start:
            timer_start = time_ns()
            print()
            log.info("Process start @ %s", strftime("%a %d %b %Y %H:%M:%S", localtime()))

            if log.getLogger().getEffectiveLevel() > log.DEBUG:
                pbar = tqdm(self.tasks, desc=":: Progress", unit="image", ncols=100)
                for task in pbar:
                    image = self.scanned_images[task]
                    image_count = self.slice(image)
                    sliced_images_count += image_count
            else:
                for task in self.tasks:
                    image = self.scanned_images[task]
                    image_count = self.slice(image)
                    sliced_images_count += image_count

            timer_stop = time_ns()
            seconds = (timer_stop - timer_start) / 1000000000
            timer_result = strftime("%H hours, %M minutes and %S seconds.", gmtime(seconds))

            log.info("Process finish @ %s", strftime("%a %d %b %Y %H:%M:%S", localtime()))
            print()
            action = "Saved"
            if self.settings["test_mode"]: action = "Detected"
            log.info("%s %s images from %s scanned images", action, sliced_images_count, len(self.tasks))
            log.info("Time lapsed: %s", timer_result)

    def list_tasks(self):
        print()
        dots = self.dots
        log.info("Listing tasks (%s)", len(self.tasks))

        if self.tasks:
            for i in range(len(self.tasks)):
                if i in [9, 99, 999, 9999, 99999, 999999]:
                    dots = dots[1:]

                log.info("TASK #%s %s %s [ID:%s]", i + 1, dots, self.scanned_images[self.tasks[i]][0], self.tasks[i])
        else:
            log.info("Task list is empty")

    def list_commands(self):
        print()
        log.info("Listing commands (%s)", len(self.cmds))
        if self.cmds:
            for cmd in self.cmds:
                log.info(cmd)
        else:
            log.info("Command list is empty")
        
    def remove_task_with_image_id(self, ids):
        removed = []

        for id in ids:
            if id in self.tasks:
                self.tasks.remove(id)
                removed.append(id)
            else:
                self.cmds.append(f"[x] Image with ID [{id}] is not on the task list")

        if removed:
            self.cmds.append(f"[-] Removed image with ID {removed} from the task list")

    def add_task_newest_n_images(self, n):
        added = []

        if n > len(self.scanned_images):
            log.error("Cannot add more than %s newest images", len(self.scanned_images))
            sys.exit()

        for i in range(n):
            id = (len(self.scanned_images) - i) - 1

            if id in self.tasks:
                self.cmds.append(f"[x] Image with ID [{id}] is already on the task list")
            else:
                self.tasks.append(id)
                added.append(id)
            
        if added:
            self.cmds.append(f"[+] Added image with ID {added} to the task list")

    def add_task_oldest_n_images(self, n):
        added = []

        if n > len(self.scanned_images):
            log.error("Cannot add more than %s oldest images", len(self.scanned_images))
            sys.exit()

        for i in range(n):
            id = i

            if id in self.tasks:
                self.cmds.append(f"[x] Image with ID [{id}] is already on the task list")
            else:
                self.tasks.append(id)
                added.append(id)

        if added:
            self.cmds.append(f"[+] Added image with ID {added} to the task list")

    def add_task_with_image_id(self, ids):
        added = []

        for id in ids:
            if id in self.tasks:
                self.cmds.append(f"[x] Image with ID [{id}] is already on the task list")
            elif not id in range(0, len(self.scanned_images)):
                self.cmds.append(f"[x] Image with ID [{id}] does not exist")
            else:
                self.tasks.append(id)
                added.append(id)

        if added:
            self.cmds.append(f"[+] Added image with ID {added} to the task list")

    def add_task_all_images(self):
        self.cmds.append("[+] Added all scanned images to task list")
        self.tasks = []

        for i in range(len(self.scanned_images)):
            self.tasks.append(i)

    def add_task_random_n_images(self, n):
        added = []
        ids = []
        pool = []

        for i in range(0, len(self.scanned_images) - 1):
            pool.append(i)

        if n > len(self.scanned_images) - 1:
            log.error("Cannot add more than %s random images", len(self.scanned_images) - 1)
            sys.exit()

        while len(ids) != n:
            pick = random.choice(pool)
            pool.remove(pick)
            ids.append(pick)

        for id in ids:
            if id in self.tasks:
                self.cmds.append(f"[x] Image with ID [{id}] is already on the task list")
            else:
                self.tasks.append(id)
                added.append(id)

        if added:
            self.cmds.append(f"[+] Added image with ID {added} to the task list")

    def list_scanned_images(self):
        print()
        dots = self.dots
        log.info("Listing scanned images (%s)", len(self.scanned_images))

        for i in range(len(self.scanned_images)):
            if i in [10, 100, 1000, 10000, 100000, 1000000]:
                dots = dots[1:]

            log.info("ID %s %s %s", i, dots, self.scanned_images[i][0])

    def list_scanned_images_to_file(self):
        print()
        dots = self.dots
        lines = []

        if sys.platform in ["win32", "cygwin"]:
            filepath = os.path.join(os.path.expanduser("~"), ".ScanImageSlicer", "list of scanned images.txt")
        else:
            filepath = os.path.join(os.path.expanduser("~"), ".config", "ScanImageSlicer", "list of scanned images.txt")

        for i in range(len(self.scanned_images)):
            if i in [10, 100, 1000, 10000, 100000, 1000000]:
                dots = dots[1:]

            lines.append(f"ID {i} {dots} {self.scanned_images[i][0]}\n")

        with open(filepath, 'w') as outfile:
            try:
                outfile.writelines(lines)
                outfile.close()
                log.info("Saved list of scanned images to file: %s", filepath)
            except OSError as e:
                sys.exit(e)

    def collect_scanned_images(self):
        scanned_images = []
        accepted_formats = [
            ".bmp",
            ".dib",
            ".jpeg",
            ".jpg",
            ".jpe",
            ".jp2",
            ".png",
            ".webp",
            ".pbm",
            ".pgm",
            ".ppm",
            ".pxm",
            ".pnm",
            ".sr",
            ".ras",
            ".tiff",
            ".tif",
            ".exr",
            ".hdr",
            ".pic"
        ]
        
        inpath = self.settings["input"]
        outpath = self.settings["output"]

        for path, dirs, files in os.walk(inpath):
            for f in files:
                abs_path = os.path.join(path, f)
                rel_path = os.path.relpath(path, inpath)
                dest_path = os.path.join(outpath, rel_path)
                ctime = os.stat(abs_path).st_ctime
                data = [f, abs_path, dest_path, rel_path, ctime]

                for suffix in accepted_formats:
                    if f.lower().endswith(suffix):
                        scanned_images.append(data)

        if scanned_images:
            log.debug("Found %s scanned images at: %s", len(scanned_images), self.settings["input"])
            scanned_images = sorted(scanned_images, key=itemgetter(4))

            for scanned_image in scanned_images:
                scanned_image.pop()

            return scanned_images
        else:
            log.error("Could not find any scanned images at: %s", self.settings["input"])
            sys.exit()


#!/usr/bin/env python3

import cv2 as cv
import imutils as im
import numpy as np

def show_images(topic, images, max_height, preview):
    action = ""
    max_height = max_height
    topic = topic + "|Space for next|Q or ESC to exit"
    index = 0
    
    for i in range(len(images)):
        images[i] = im.resize(images[i], height=min(max_height, images[i].shape[0]))

    while(True):
        cv.imshow(topic, images[index])

        key = cv.waitKey(20)

        if key in [27, 113]:
            action = "exit"
            break
        elif key == 32:
            break
        elif key == 115:
            if preview:
                action = "save"
                break
        elif key == 102:
            if index < len(images) - 1:
                index += 1
            else:
                index = 0
    
    cv.destroyAllWindows()
    return action

def bcg(image, brightness, contrast, gamma):
    lut = np.empty((1, 256), np.uint8)

    for i in range(256):
        lut[0, i] = np.clip(pow(i / 255.0, gamma) * 255.0, 0, 255)

    result = cv.convertScaleAbs(image, alpha=contrast, beta=brightness)
    result = cv.LUT(result, lut)

    return result

def denoise(image, strength):
    s = [3, 3]

    if strength != 0:
        if strength == 1:
            s.append(3)
            s.append(3)
        elif strength == 2:
            s.append(5)
            s.append(7)
        elif strength == 3:
            s.append(7)
            s.append(11)
        elif strength == 4:
            s.append(9)
            s.append(15)
        elif strength == 5:
            s.append(11)
            s.append(19)

        result = cv.fastNlMeansDenoisingColored(image, None, s[0], s[1], s[2], s[3])
        return result